#!/usr/bin/perl -Tw
#
# LMS version 1.8.1 Shyla _ moded by ReYu
#
#  (C) 2001-2010 LMS Developers
#
#  Please, see the doc/AUTHORS for more information about authors!
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License Version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
#  USA.
#
#  $Id: lms-cashimport-Inteligo,v 1.1.2.2 2006/01/16 09:35:17 reyu Exp $

use strict;
use DBI;
use Config::IniFiles;
use Getopt::Long;
use HTTP::Cookies;
use HTTP::Request::Common qw(POST GET);
use Crypt::SSLeay;
use LWP::UserAgent;
use POSIX;
use POSIX qw(mktime strftime);
use HTML::TokeParser;
use Time::Local;
use vars qw($configfile $quiet $help $version);
use Digest::MD5;
use Text::Iconv;
use Digest::MD5 qw(md5_hex);

my $_version = '1.8.1 Shyla moded by Simson';

my %options = (
		"--config-file|C=s"	=>	\$configfile,
		"--quiet|q"		=>	\$quiet,
		"--help|h"		=>	\$help,
		"--version|v"		=>	\$version,
		);

Getopt::Long::config("no_ignore_case");
GetOptions(%options);

if($help)
{
	print STDERR <<EOF;
	lms-cashimport-inteligo, version $_version
		(C) 2001-2010 LMS Developers & ReYu & Simson

	-C, --config-file=/etc/lms/lms.ini	alternate config file (default: /etc/lms/lms.ini);
	-h, --help			print this help and exit;
	-v, --version			print version info and exit;
	-q, --quiet			suppress any output, except errors;

EOF
	exit 0;
}

if($version)
{
	print STDERR <<EOF;
	lms-cashimport-inteligo, version $_version
		(C) 2001-2011 LMS Developers & ReYu & Simson

EOF
	exit 0;
}

if(!$configfile)
{
	$configfile = "/etc/lms/lms.ini";
}

if(!$quiet)
{
	print STDOUT "lms-cashimport-inteligo, version $_version\n";
	print STDOUT "(C) 2001-2011 LMS Developers & ReYu & Simson\n";
	print STDOUT "Using file $configfile as config.\n";
}

if(! -r $configfile)
{
	print STDERR "Fatal error: Unable to read configuration file $configfile, exiting.\n";
	exit 1;
}

my $ini = new Config::IniFiles -file => $configfile;
print @Config::IniFiles::errors;

my $dbtype = $ini->val('database', 'type') || 'mysql';
my $dbhost = $ini->val('database', 'host') || 'localhost';
my $dbuser = $ini->val('database', 'user') || 'root';
my $dbpasswd = $ini->val('database', 'password') || '';
my $dbname = $ini->val('database', 'database') || 'lms';

my $login_i = $ini->val('cashimport_inteligo', 'id') || '12345678';
my $passwd_i = $ini->val('cashimport_inteligo', 'pass') ||  'abcdef';
my $konto = $ini->val('cashimport_inteligo', 'account') || '111111111111';
my $time_y_start = $ini->val('cashimport_inteligo', 'wyciag_r') || strftime "%Y", localtime(time-86400);
my $time_m_start = $ini->val('cashimport_inteligo', 'wyciag_m') || strftime "%m", localtime(time-86400);
my $time_d_start = $ini->val('cashimport_inteligo', 'wyciag_d') || strftime "%d", localtime(time-86400);
my $time_y_end = $ini->val('cashimport_inteligo', 'wyciag_r') || strftime "%Y", localtime(time);
my $time_m_end = $ini->val('cashimport_inteligo', 'wyciag_m') || strftime "%m", localtime(time);
my $time_d_end = $ini->val('cashimport_inteligo', 'wyciag_d') || strftime "%d", localtime(time);
my $trans_type = $ini->val('cashimport_inteligo', 'trans_type') || 'CREDIT';
my $file_out_dir = $ini->val('cashimport_inteligo', 'history_dir') || '/var/lms/cashimport';

my $dbase;

if($dbtype eq "mysql")
{
	$dbase = DBI->connect("DBI:mysql:database=$dbname;host=$dbhost","$dbuser","$dbpasswd", { RaiseError => 1 });
}
elsif($dbtype eq "postgres")
{
	$dbase = DBI->connect("DBI:Pg:dbname=$dbname;host=$dbhost","$dbuser","$dbpasswd", { RaiseError => 1 });
}
else
{
	print STDERR "Fatal error: unsupported database type: $dbtype, exiting.\n";
	exit 1;
}

# Definicja Agenta http
my $sd ='';
my $cookie_jar = HTTP::Cookies->new;
my $ua = new LWP::UserAgent;
$ua->timeout(30);
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.7.8) Gecko/20050511 Firefox/1.0.4');
$ua->env_proxy();

# Wchodzimy na stron
my $get = new HTTP::Request(GET => 'https://inteligo.pl/secure');
my $res = $ua->request($get);
if ($res->content =~ /Przepraszamy/ ){
	print "ERROR: System chwilowo niedostpny", "\n";
	exit 1;
}		
if ($res->content =~ /Identyfikator/ ){
	my $p = HTML::TokeParser->new(\$res->content);
	while ( my $token1 = $p->get_tag("input")){
		if (my $ur = $token1->[1]{name} =~ /sd/){
			my $url = $token1->[1]{name} || "-";
			my $url2 = $token1->[1]{value} || "-";
			$sd = $url2;
		}
	} 
}		
print "Logujemy do Serwisu ... \t";
my $post = POST 'https://inteligo.pl/secure', [
sd => $sd,
   form_name => "login", 
   btn_ok => "",
   menu => "",
   cif => $login_i,
   password => $passwd_i,
   btn_ok => "", 
   ];
my $res1 = $ua->request($post);
if ($res1->content =~ /Stan Twoich/ ){
	print "Jestesmy zalogowani", "\n";
	my $p = HTML::TokeParser->new(\$res1->content);
	while ( my $token1 = $p->get_tag("input")){
		if (my $ur = $token1->[1]{name} eq "sd"){
			my $url = $token1->[1]{name} || "-";
			my $url2 = $token1->[1]{value} || "-";
			$sd = $url2;
		}
	}
}
print "Przechodzimy do historii ...\t";
my $post1 = POST 'https://inteligo.pl/secure/', [
sd => $sd,
   section => "mojerachunki",
   return_point => "1", 
   form_name => "accounts",
   menu => "history",
   button => "",
   ];
$res = $ua->request($post1);
if ($res->content =~ /Historia rachunku/ ){
	print "Strona Historii Rachunku \n";
	my $p1 = HTML::TokeParser->new(\$res->content);
	while ( my $token2 = $p1->get_tag("input")){
		if (my $ur = $token2->[1]{name} eq "sd"){
			my $url = $token2->[1]{name} || "-";
			my $url2 = $token2->[1]{value} || "-";
			$sd = $url2;
		}
	}
}
print "Zmieniamy filtr historii....\t";
my $post2 = POST 'https://inteligo.pl/secure', [
sd => $sd,
   section => "mojerachunki", 
   return_point => "1", 
   form_name => "history",
   menu => "",
   button => "filter",
   selected_acc => $konto,
   beg_date_y => $time_y_start,
   beg_date_m => $time_m_start,
   beg_date_d => $time_d_start,
   end_date_y => $time_y_end,
   end_date_m => $time_m_end,
   end_date_d => $time_d_end,
   sel_tx_filter_type => $trans_type,
   search_text => "",
   search_type => "all_words",
   search_account => "",
   amount_min => '',
   amount_max => '',
   export_type => 'CSV',
   ];
$res = $ua->request($post2);
if ($res->content =~ /Historia rachunku/ ){
	print "Zmieniono filtr $time_d_start.$time_m_start.$time_y_start - $time_d_end.$time_m_end.$time_y_end\n";
	my $p1 = HTML::TokeParser->new(\$res->content);
	while (my $token2 = $p1->get_tag("input")){
		if (my $ur = $token2->[1]{name} eq "sd"){
			my $url = $token2->[1]{name} || "-";
			my $url2 = $token2->[1]{value} || "-";
			$sd = $url2;
		}
	}
}		 
print "Pobieramy plik historii....\t";
my $post3 = POST 'https://inteligo.pl/secure', [
sd => $sd,
   section => "mojerachunki",
   return_point => "1", 
   form_name => "history",
   menu => "",
   button => "export_list",
   selected_acc => $konto,
   beg_date_y => $time_y_start,
   beg_date_m => $time_m_start,
   beg_date_d => $time_d_start,
   end_date_y => $time_y_end,
   end_date_m => $time_m_end,
   end_date_d => $time_d_end,
   sel_tx_filter_type => $trans_type,
   search_text => "",
   search_type => "all_words",
   search_account => "",
   amount_min => "",
   amount_max => "",
   export_type => "CSV",
   ];
$res = $ua->request($post3);
my $post4 = 'https://inteligo.pl' . $res->header('Location');
$post4 =GET $post4;
$res =$ua->request($post4);
print "Pobrano plik\n";
my @dana = split("\n", $res->content); 
my $file_name = $file_out_dir."/". $konto ."_". time();
if ($file_out_dir){
	open (DOUT,">$file_name");
	print DOUT "Ilosc pozycji". scalar(@dana) ."\n";
}

my $okCounter = 0;

for (my $x=1;$x<scalar(@dana);$x++){
#my $converter = Text::Iconv->new("UTF-8", "ISO-8859-2");
	my $line = $dana[$x];
	$line =~ s/"//g; #"
	$line =~ s/'//g; #"
#	$line = $converter->convert($line);

print "\n$line\n";
	my @lines = split(",",$line);
	my @da = split('-',$lines[2]);

#my $date = mktime(0,0,0,$da[2],$da[1]-1, $da[0]-1900);
	my $statementid = $lines[0];
	my $date = $lines[2];
	my $value = $lines[4];
	my $account = $lines[7];
	my $sender = $lines[8];
	my $description = $lines[11];
	my $customerid = 0;
	my $customer = "BRAK!";
	my $fullDescription = $sender."<BR/>\n".$description."<BR/>\n".$account."<BR/>\n";

	if ($file_out_dir){ 
		print DOUT $line ."\n";
	}
	if($account) {
		my $sdbq = $dbase->prepare("SELECT id, lastname, name, address FROM customers WHERE notes LIKE '%$account%'");
		$sdbq->execute;
		if (my $row = $sdbq->fetchrow_hashref()){
			$customerid = $row->{'id'};
			$customer = $row->{'lastname'}." ".$row->{'name'}." ".$row->{'address'};
		}
	}

	my $hash = "BankStatementId:".$statementid;
	my $sdbq = $dbase->prepare("SELECT id FROM cashimport WHERE hash = '$hash'");
	$sdbq->execute();
	my $row = $sdbq->fetchrow_hashref();
	if ($row->{'id'}) {
		print DOUT "ERROR: Hash exists, ignoring $customer $sender", "\n";
	} elsif ($customerid == '0') {
		$sdbq = $dbase->prepare("INSERT INTO cashimport ( Date , Value , Customer , Description , CustomerId , Hash ) VALUES (UNIX_TIMESTAMP('$date'), '$value', '$customer', '$fullDescription', '$customerid', '$hash');");
		$sdbq->execute();
        print "Data: $date\nWysa: $sender\nAutomatyczna identyfikacja klienta: $customer ($customerid)\nNr konta:$account\nWarto: $value\nOpis: $description\nHash: $hash\n";
		$okCounter ++;
	} else {
        print "\nINSERT INTO cash ( time , userid , type , value , customerid , comment ) VALUES (UNIX_TIMESTAMP('$date'), '2', '1', '$value', '$customerid', '$fullDescription' );\n";
        $sdbq = $dbase->prepare("INSERT INTO cash ( time , userid , type , value , customerid , comment ) VALUES (UNIX_TIMESTAMP('$date'), '2', '1', '$value', '$customerid', '$fullDescription' );");
        $sdbq->execute();

        $sdbq = $dbase->prepare("INSERT INTO cashimport ( Date , Value , Customer , Description , CustomerId , Hash , Closed ) VALUES (UNIX_TIMESTAMP('$date'), '$value', '$customer', '$fullDescription', '$customerid', '$hash', '1');");
        $sdbq->execute();
        print "Data: $date\nWysa: $sender\nAutomatyczna identyfikacja klienta: $customer ($customerid)\nNr konta:$account\nWarto: $value\nOpis: $description\nHash: $hash\n";
		print "\nCustomerID znaleziony! Wpata zaksigowana automatycznie!\n";
        $okCounter ++;
	}
}
close DOUT;

print "OK: $okCounter\n";
if ($okCounter eq 0) {
	exit 2;
}
exit 0;

print "ERROR: Nieznany bd";
exit 1;

